#!/usr/bin/perl
use Gtk2 '-init';
use strict;
use warnings;

use FindBin qw($Bin);
use lib $Bin;

use constant SLASH => ($^O  eq 'MSWin32')? '\\' : '/';
my $slash="\\".SLASH;
use constant
{
 TRUE  => 1, FALSE => 0,

 SONG_FILE	=> 0,	SONG_PATH	=> 1,	SONG_MODIF	=> 2,
 SONG_LENGTH	=> 3,	SONG_SIZE	=> 4,	SONG_BITRATE	=> 5,
 SONG_FORMAT	=> 6,	SONG_CHANNELS	=> 7,	SONG_SAMPRATE	=> 8,
 SONG_TITLE	=> 9,	SONG_ARTIST	=> 10,	SONG_ALBUM	=> 11,
 SONG_DISC	=> 12,	SONG_TRACK	=> 13,	SONG_DATE	=> 14,
 SONG_VERSION	=> 15,	SONG_GENRE	=> 16,	SONG_COMMENT	=> 17,
 SONG_AUTHOR	=> 18,
 SONG_ADDED	=> 19,	SONG_LASTPLAY	=> 20,	SONG_NBPLAY	=> 21,
 SONG_RATING	=> 22,	SONG_FLAGS	=> 23,
 SONG_MISSINGSINCE => 24,
 SONGLAST => 24,	# nb of last field

};

BEGIN { require 'gmusicbrowser_tags.pm'; }

our @TAGSREADFROMTAGS=(SONG_TITLE,SONG_ARTIST,SONG_DATE,SONG_ALBUM,SONG_COMMENT,SONG_TRACK,SONG_GENRE,SONG_DISC,SONG_VERSION);

our %TAGNB=
(	Filename	=> SONG_FILE,
	Pathname	=> SONG_PATH,
	Modification	=> SONG_MODIF,
	Title		=> SONG_TITLE,
	Artist		=> SONG_ARTIST,
	Date		=> SONG_DATE,
	Album		=> SONG_ALBUM,
	Comment		=> SONG_COMMENT,
	Track		=> SONG_TRACK,
	Genre		=> SONG_GENRE,
	Length		=> SONG_LENGTH,
	Size		=> SONG_SIZE,
	Disc		=> SONG_DISC,
	Version		=> SONG_VERSION,
	Rating		=> SONG_RATING,
	Added		=> SONG_ADDED,
	'Last played'	=> SONG_LASTPLAY,
	'Nb played'	=> SONG_NBPLAY,
	Flags		=> SONG_FLAGS,
	Bitrate		=> SONG_BITRATE,
	Type		=> SONG_FORMAT,
);

our @TagProp;	# TagProp[columnnumber]=[name,data_type]
# names : use %TAGNB
$TagProp[ $TAGNB{$_} ]=[$_] for keys %TAGNB;
# data_type :	d : date in seconds since epoch (01-01-1970)
#		s : string
#		n : number
#		l : length in seconds
#		f : flags (string of one-letter flags)
#		g : genres (~flags) FIXME
$TagProp[$_][1]='d' for (SONG_MODIF,SONG_ADDED,SONG_LASTPLAY);
$TagProp[$_][1]='s' for (SONG_FILE,SONG_PATH,SONG_TITLE,SONG_ARTIST,SONG_ALBUM,SONG_COMMENT,SONG_VERSION,SONG_FORMAT);
$TagProp[$_][1]='n' for (SONG_DATE,SONG_TRACK,SONG_DISC,SONG_NBPLAY,SONG_RATING,SONG_SIZE,SONG_BITRATE);
$TagProp[SONG_LENGTH][1]='l';
$TagProp[SONG_FLAGS][1]='f';
$TagProp[SONG_GENRE][1]='g';

sub SortList {}
sub SetWSize {}

our $debug=1;
our @Songs;

my $window = Gtk2::Window->new;
$window->signal_connect (destroy => sub {exit});
$window->set_default_size (500, 600);
my $vbox=Gtk2::VBox->new;
my $edittag;

our %Options;
if ($ARGV[0] eq '-itunes')
{	shift;
	$Options{TAG_use_latin1_if_possible}=1;
	$Options{TAG_no_desync}=1;
}


if (@ARGV>1)
{	for my $filename (@ARGV)
	{	warn $filename if $debug;
		my ($dir,$file)=$filename=~m/^(.*)$slash([^$slash]+)$/o;
		my @song;
		@song[SONG_FILE,SONG_PATH,SONG_ADDED]=($file,$dir,time);
		push @Songs,\@song;
		ReadTag::Read(\@song,$filename,$debug*2);
		#warn "@song" if $debug;
	}
	$edittag=MassTag->new($window,0..$#Songs);
}
elsif (@ARGV==1)
{	my $filename=shift;
	my ($dir,$file)=$filename=~m/^(.*)$slash([^$slash]+)$/o;
	my @song;
	@song[SONG_FILE,SONG_PATH,SONG_ADDED]=($file,$dir,time);
	@Songs=(\@song);
	$edittag=EditTag->new($window,0);
}
else
{	warn "usage : edittag.pl [FILE...]\n";
	exit;
}
$window->add ($vbox);
$vbox->add($edittag);
my $butok=Gtk2::Button->new('OK');
my $butcancel=Gtk2::Button->new('Cancel');
$vbox->pack_start($butok,FALSE,FALSE,0);
$vbox->pack_start($butcancel,FALSE,FALSE,0);
$butok->signal_connect (clicked=> sub
	{	$edittag->save;
		exit;
	});
$butcancel->signal_connect (clicked=> sub {exit} );
$window->show_all;

Gtk2->main;

sub NewIconButton
{	my ($icon,$text,$coderef,$style)=@_;
	my $but=Gtk2::Button->new;
	$but->set_relief($style) if $style;
	my $widget=Gtk2::Image->new_from_stock($icon,'menu');
	if ($text)
	{	my $box=Gtk2::HBox->new(FALSE, 6);
		$box->pack_start($_, FALSE, FALSE, 2)
			for ($widget,Gtk2::Label->new($text));
		$widget=$box;
	}
	$but->add($widget);
	$but->signal_connect('clicked' => $coderef) if $coderef;
	return $but;
}
sub PixLoader_callback
{	my ($self,$w,$h,$max,$force)=@_;
	if ($force or $w>$max or $h>$max)
	{	my $r=$w/$h;
		if ($r>1) {$h=int(($w=$max)/$r);}
		else	  {$w=int(($h=$max)*$r);}
		$self->set_size($w,$h);
	}
}

sub ChoosePix
{	my ($path,$text)=@_;
	my $PixSelector=Gtk2::FileSelection->new ($text||'Choose Picture');

	$PixSelector->add_button('no picture','reject'); #FIXME add before ok and cancel buttons
	$PixSelector->set_filename($path.SLASH) if $path;
	$PixSelector->complete ('*.jpg');	#FIXME
	my $response = $PixSelector->run;
	my $ret;
	if ($response eq 'ok')
	{	$ret=Glib->filename_from_unicode( $PixSelector->get_filename);
		unless (-r $ret) { warn "can't read $ret\n"; $ret=undef; }
	}
	elsif ($response eq 'reject') {$ret=''}
	else {$ret=undef}
	$PixSelector->destroy;
	return $ret;
}

sub ChooseSaveFile
{	my ($window,$msg,$file) = @_;
	my $dialog=Gtk2::FileChooserDialog->new($msg,$window,'save',
					'gtk-ok' => 'ok',
					'gtk-cancel' => 'none',
					);
	$dialog->set_current_name($file);

	if ($dialog->run eq 'ok')
	{   $file=$dialog->get_filename;
	    $file=Glib->filename_from_unicode($file);
	    #check if exist
	}
	else {$file=undef}
	$dialog->destroy;
	if ($file && -f $file)
	{	my $dialog = Gtk2::MessageDialog->new
		( $window,
		  [qw/modal destroy-with-parent/],
		  'warning','ok-cancel',
		  "'$file' exists. Overwrite ?"
		);
		$dialog->show_all;
		if ($dialog->run ne 'ok') {$file=undef;}
		$dialog->destroy;
	}
	return $file;
}

sub EditLyricsDialog
{	my ($window,$init,$text,$sub)=@_;
	my $dialog = Gtk2::Dialog->new ($text||'Edit Lyrics', $window,
				'destroy-with-parent',
				#[qw/modal destroy-with-parent/],
				'gtk-ok' => 'ok',
				'gtk-cancel' => 'none');
	$dialog->set_default_response ('ok');
	my $textview=Gtk2::TextView->new;
	my $buffer=$textview->get_buffer;
	$buffer->set_text($init);

	my $sw=Gtk2::ScrolledWindow->new(undef, undef);
	$sw->set_shadow_type('etched-in');
	$sw->set_policy('automatic', 'automatic');
	$sw->add_with_viewport($textview);
	$dialog->vbox->add($sw);
	$dialog->show_all;
	$dialog->signal_connect( response => sub
		{	warn "EditLyrics response : @_\n" if $debug;
			my $response=$_[1];
			my $lyrics;
			$lyrics=$buffer->get_text( $buffer->get_bounds, 1) if $response eq 'ok';
			$dialog->destroy;
			&$sub($lyrics);
		});
	return $dialog;
}

sub Retry_Dialog
{	my ($err,$win)=@_;
	my $dialog = Gtk2::MessageDialog->new
	( $win || $window,
	  [qw/modal destroy-with-parent/],
	  'error','ok-cancel',
	  "$err\n retry ?"
	);
	$dialog->show_all;
	my $ret=($dialog->run eq 'ok')? 1 : undef;
	$dialog->destroy;
	return $ret;
}
