package GMB::Plugin::AudioScrobbler;
use strict;
use warnings;
use constant
{	CLIENTID => 'gmb', VERSION => '0.1',
	PID => 'AUDIOSCROBBLER',#used to identify the plugin and its options
};
use POSIX 'strftime';
use Digest::MD5 'md5_hex';
require 'simple_http.pm';

$::Plugins{&PID}=__PACKAGE__;

my $self=bless {},__PACKAGE__;
my $sec;
my $lasttime;
my @ToSubmit;
my $interval=1; my ($timeout,$waiting);
my ($HandshakeOK,$submiturl,$challenge);
my ($HSerrors,$Serrors,$updateurl,$Stop);
my $Log=Gtk2::ListStore->new('Glib::String');

sub init
{	::Watch($self,'SongID',\&SongChanged);
	::Watch($self,'Time',\&TimeChanged);
	$self->{on}=1;
	SongChanged();
}
sub end
{	::UnWatch($self,'SongID');
	::UnWatch($self,'Time');
	$self->{on}=undef;
	@ToSubmit=();
}
sub name {'last.fm'}

sub prefbox
{	my $vbox=Gtk2::VBox->new(::FALSE, 2);
	my $sg1=Gtk2::SizeGroup->new('horizontal');
	my $sg2=Gtk2::SizeGroup->new('horizontal');
	my $entry1=::NewPrefEntry('PLUGIN_'.PID.'_USER','username :',\&userpass_changed,$sg1,$sg2);
	my $entry2=::NewPrefEntry('PLUGIN_'.PID.'_PASS','password :',\&userpass_changed,$sg1,$sg2,1);
	my $label1=Gtk2::Label->new('last.fm plugin v'.VERSION);
	my $label2=Gtk2::Button->new('(see http://www.last.fm)');
	$label2->set_relief('none');
	$label2->signal_connect(clicked => sub
		{	my $url='http://www.last.fm';
			my $user=$::Options{'PLUGIN_'.PID.'_USER'};
			$url.="/user/$user/" if defined $user && $user ne '';
			::openurl($url);
		});
	$vbox->pack_start($_,::FALSE,::FALSE,0) for $label1,$label2,$entry1,$entry2;
	$vbox->add( ::LogView($Log) );
	if ($updateurl)
	{	my $url=Gtk2::Label->new('plugin update at '.$updateurl);
		$url->set_selectable(::TRUE);
		$vbox->pack_start($url,::FALSE,::FALSE,0);
	}
	return $vbox;
}
sub userpass_changed
{	$HandshakeOK=$HSerrors=$Serrors=$Stop=undef;
}

sub SongChanged
{	$sec=undef;
	return unless defined $::SongID;
	$sec=$::Songs[$::SongID][::SONG_LENGTH];
	$sec=	($sec<30) ?	undef :
		($sec>480)?	240 :
		$sec/2;
}

sub TimeChanged
{	if (defined $sec && defined $::PlayTime && $::PlayTime > $sec)
	{	$sec=undef;
		return if $lasttime && $::StartTime==$lasttime;
		return if $::StartedAt;
		$lasttime=$::StartTime;
		my ($title,$artist,$album,$length)
			=@{$::Songs[$::SongID]}[::SONG_TITLE,::SONG_ARTIST,::SONG_ALBUM,::SONG_LENGTH];
		return if $title eq '';
		return if $artist eq '<Unknown>';
		$album='' if $album=~m/^<Unknown>/;
		my $time=strftime('%Y-%m-%d %H:%M:%S',gmtime($lasttime));
		push @ToSubmit,[ $artist,$title,$album,'',$length,$time ];
		Sleep();
	}
}

sub Handshake
{	$HandshakeOK=0;
	my $user=$::Options{'PLUGIN_'.PID.'_USER'};
	return 0 unless defined $user && $user ne '';
	my $response_cb=
	sub
	{my ($response,@lines)=@_;
	 my $error;
	 if	(!defined $response)		{$interval*=2 unless $interval>60;$error='connection failed';}
	 elsif	($response eq 'UPTODATE')	{$updateurl=undef}
	 elsif	($response=~m/^UPDATE (.*)$/)	{$updateurl=$1}
	 elsif	($response=~m/^FAILED (.*)$/)	{$error=$1}
	 elsif	($response eq 'BADUSER')	{$error="Bad user ($user)";	$Stop=1;}
	 else					{$error='unknown error';	$Stop=1;}

	 m/^INTERVAL (\d+)$/ and $interval=$1	for @lines;

	 if (defined $error)
	 {	$HSerrors++; $Stop=1 if $HSerrors>3;
		$error.=" (retry in $interval s)" unless $Stop;
		Log('Handshake failed : '.$error);
	 }
	 else
	 {	($challenge,$submiturl)=@lines;
		$HandshakeOK=1;
		$HSerrors=0;
		Log('Handshake OK');
	 }
	};

	Send($response_cb,'http://post.audioscrobbler.com/?hs=true&p=1.1&c='.CLIENTID.'&v='.VERSION.'&u='.$user);
}

sub Submit
{	return unless @ToSubmit;
	my $user=$::Options{'PLUGIN_'.PID.'_USER'};
	my $pass=$::Options{'PLUGIN_'.PID.'_PASS'};
	my $post="u=$user&s=".md5_hex(md5_hex($pass).$challenge);
	my $i=0;
	while ($ToSubmit[$i])
	{	my @data=@{$ToSubmit[$i]};
		use bytes;
		s/([^A-Za-z0-9-])/sprintf('%%%02X', ord($1))/seg for @data;
		$post.=sprintf "&a[$i]=%s&t[$i]=%s&b[$i]=%s&m[$i]=%s&l[$i]=%s&i[$i]=%s",@data;
		$i++;
		last if $i==10; #don't submit more than 10 songs at a time
	}
	my $response_cb=sub
	{	my ($response,@lines)=@_;
		my $error;
		if	(!defined $response) {$interval*=2 unless $interval>60;$error='connection failed';}
		elsif	($response eq 'OK')
		{	$Serrors=0;
			Log('Submit OK '.($i>1 ? "($i songs)" : "($ToSubmit[0][1] by $ToSubmit[0][0])"));
			splice @ToSubmit,0,$i;
		}
		elsif	($response eq 'BADAUTH')
		{	$error='Bad Password';
			$HandshakeOK=0;
			$Serrors++;
		}
		elsif	($response=~m/^FAILED (.*)$/)
		{	$error=$1;
			$Serrors++;
		}
		else	{$error='unknown error';$HandshakeOK=0;$Stop=1;}

		m/^INTERVAL (\d+)$/ and $interval=$1	for @lines;
		$Stop=1 if $Serrors && $Serrors>3;

		if (defined $error)
		{	$error.=" (retry in $interval s)" unless $Stop;
			Log('Submit failed : '.$error);
		}
	};

	warn "submitting: $post\n" if $::debug;
	Send($response_cb,$submiturl,$post);
}

sub Sleep
{	return unless $self->{on};
	return if $waiting || $timeout;
	$timeout=Glib::Timeout->add(1000*$interval,\&Awake) if @ToSubmit;
}
sub Awake
{	#warn "Awoke\n";
	$timeout=undef;
	return 0 unless $self->{on};
	if ($HandshakeOK)	{ Submit(); }
	else			{ Handshake(); }
	Sleep() if !$waiting && @ToSubmit && !$Stop;
	return 0;
}
sub Send
{	my ($response_cb,$url,$post)=@_;
	my $cb=sub
	{	my @response=(defined $_[0])? split "\012",$_[0] : ();
		$waiting=undef;
		&$response_cb(@response);
		Sleep() unless $Stop;
	};
	$waiting=Simple_http::get_with_cb(cb => $cb,url => $url,post => $post);
	#&$cb() unless $waiting;
}

sub Log
{	my $text=$_[0];
	$Log->set( $Log->prepend,0, localtime().'  '.$text );
	warn "$text\n" if $::debug;
	if (my $iter=$Log->iter_nth_child(undef,50)) { $Log->remove($iter); }
}

1;
